﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading.Tasks;

/*
 *         Progetto: Progetto "Mappa Meteo"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 16/12/2016
 *  
 *      Descrizione: Routine di gestione della cache dei dati meteo. 
 *                   La struttura dati contenente le informazioni sulle città può essere memorizzata
 *                   in un file per rendere più veloci i caricamenti successivi.
 *                   
 *                   La validità della cache è pari a un giorno.
 *                   
 */

namespace MappaMeteo
{
    public static class Cache
    {
        static Cache()
        {
            try
            {
                if (!Directory.Exists(NomeDirectory))
                    Directory.CreateDirectory(NomeDirectory);
            }
            catch (Exception) { }
        }

        public static string NomeDirectory
        {
            get
            {
                return Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Cache");
            }
        }

        public static string NomeFile
        {
            get
            {

                return Path.Combine(NomeDirectory, String.Format("{0:yyyy-MM-dd}.cache", DateTime.Now));
            }
        }

        public static bool OggettoInCache()
        {
            return File.Exists(NomeFile);
        }

        public static void Salva(Object o)
        {
            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream(NomeFile, FileMode.Create, FileAccess.Write, FileShare.None);
            try
            {
                formatter.Serialize(stream, o);
            }
            finally
            {
                stream.Close();
            }
        }

        public static Object Carica()
        {
            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream(NomeFile, FileMode.Open, FileAccess.Read, FileShare.Read);
            try
            {
                return formatter.Deserialize(stream);
            }
            finally
            {
                stream.Close();
            }
        }

        public static void Pulisci()
        {
            foreach (var s in Directory.GetFiles(NomeDirectory))
                File.Delete(s);
        }

    }
}
