﻿using GMap.NET;
using GMap.NET.MapProviders;
using GMap.NET.WindowsPresentation;
using System;
using System.Collections.Generic;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

/*
 *         Progetto: Progetto "Mappa Meteo"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 16/12/2016
 *  
 *      Descrizione: Routine di gestione della finestra principale 
 *                   dell'applicazione WPF.
 *                   
 */

namespace MappaMeteo
{
    /// <summary>
    /// Logica di interazione per MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public const string Nazione = "ITALY";
        List<System.Windows.Point> zonaRitaglio;

        public MainWindow()
        {
            InitializeComponent();
            //pbCarica.Visibility = System.Windows.Visibility.Collapsed;
            mappa.DragButton = MouseButton.Left;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            mappa.MapProvider = GMapProviders.OpenStreetMap;
            GMap.NET.GMaps.Instance.Mode = GMap.NET.AccessMode.ServerOnly;
            mappa.Position = new PointLatLng(41.89193, 12.51133);        // Roma

            zonaRitaglio = new List<Point>();

            // Italia

            zonaRitaglio.Add(new Point(6.7, 46.0));
            zonaRitaglio.Add(new Point(7.5, 43.6));
            zonaRitaglio.Add(new Point(10.0, 43.8));
            zonaRitaglio.Add(new Point(10.3, 41.4));
            zonaRitaglio.Add(new Point(7.8, 41.1));
            zonaRitaglio.Add(new Point(8.2, 38.7));
            zonaRitaglio.Add(new Point(15.1, 36.4));
            zonaRitaglio.Add(new Point(18.9, 40.4));
            zonaRitaglio.Add(new Point(13.3, 45.1));
            zonaRitaglio.Add(new Point(13.7, 46.8));
            zonaRitaglio.Add(new Point(10.1, 47.0));

            Wait.Visibility = Visibility.Collapsed;
        }

        private void AggiornaMarker(StrutturaDati sd)
        {
            lbData.Visibility = Visibility.Visible;
            lbData.Content = sd.Data.ToString("dd/MM/yyyy HH:mm:ss", CultureInfo.InvariantCulture); ;
            mappa.Markers.Clear();
            foreach (var c in sd.Citta)
            {
                var pos = new PointLatLng(c.Lat, c.Long);
                GMapMarker m = new GMapMarker(pos);

                if (rbTemp.IsChecked == true)
                {
                    //Mappa delle temerature
                    string fg = c.TemperaturaCelsius == sd.TemperaturaMax || c.TemperaturaCelsius == sd.TemperaturaMin ? "White" : "Black";
                    string bg = c.TemperaturaCelsius == sd.TemperaturaMax ? "Red" : (c.TemperaturaCelsius == sd.TemperaturaMin ? "Blue" : "Yellow");
                    var r = new Riquadro(c.TemperaturaCelsius.ToString(), fg, bg);
                    r.ToolTip = c.Localita;

                    m.Shape = r;
                    m.Offset = new System.Windows.Point(-r.Width / 2, -r.Height / 2);
                }
                else if (rbMeteo.IsChecked == true)
                {
                    // Mappa meteo
                    System.Windows.Controls.Image img = new System.Windows.Controls.Image();
                    img.Stretch = System.Windows.Media.Stretch.UniformToFill;
                    img.Width = 50;
                    img.Height = 50;


                    var bmp = (System.Drawing.Bitmap)Properties.Resources.ResourceManager.GetObject(String.Format("_{0}", c.MeteoIcona));
                    if (bmp != null)
                    {
                        MemoryStream ms = new MemoryStream();
                        bmp.Save(ms, ImageFormat.Png);
                        ms.Position = 0;
                        BitmapImage bi = new BitmapImage();
                        bi.BeginInit();
                        ms.Seek(0, SeekOrigin.Begin);
                        bi.StreamSource = ms;
                        bi.EndInit();
                        img.Source = bi;

                    }

                    m.Shape = img;
                    m.Offset = new System.Windows.Point(-img.Width / 2, -img.Height / 2);
                }

                mappa.Markers.Add(m);
            }

            mappa.Position = new PointLatLng(sd.LatMedia, sd.LongMedia);
        }

        private async Task SvuotaCacheAsync()
        {
            await Task.Run(() =>
            {

                Cache.Pulisci();

                tbInfo.Dispatcher.Invoke(() =>
                {
                    tbInfo.Text = "Cache svuotata";
                });

            });
        }

        private async Task AggiornaAsync()
        {
            StrutturaDati sd = null;

            await Task.Run(async () =>
            {
                mappa.Dispatcher.Invoke(() => { mappa.Markers.Clear(); });
                lbData.Dispatcher.Invoke(() => { lbData.Visibility = Visibility.Hidden; });
                sd = await StrutturaDati.GeneraAsync(zonaRitaglio, tbInfo, Wait);
                Cache.Salva(sd);
            });

            AggiornaMarker(sd);
        }
        private async void btnSvuotaCache_Click(object sender, RoutedEventArgs e)
        {
            await SvuotaCacheAsync();
        }

        private async void btnAggiorna_Click(object sender, RoutedEventArgs e)
        {
            ((Storyboard)FindResource("WaitStoryboard")).Begin();
            await AggiornaAsync();
            ((Storyboard)FindResource("WaitStoryboard")).Stop();
        }

        private void btnCambiaAPIKey_Click(object sender, RoutedEventArgs e)
        {
            InputApiKey.Mostra();
        }

        private void btnCambiaAPIKey_Initialized(object sender, System.EventArgs e)
        {
            if (String.IsNullOrWhiteSpace(Properties.Settings.Default.ApiKey))
            {
                if (InputApiKey.Mostra() == false)
                    Environment.Exit(1);
            }

        }

        private void rbTemp_Checked(object sender, RoutedEventArgs e)
        {
            btnAggiorna_Click(sender, e);
        }

        private void rbMeteo_Checked(object sender, RoutedEventArgs e)
        {
            btnAggiorna_Click(sender, e);
        }
    }
}
