﻿using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WeatherNet;
using WeatherNet.Model;
using WeatherNet.Util.Api;

/*
*         Progetto: Progetto "Mappa Meteo"
* 
*           Autore: Roberto Fuligni
*  Ultima modifica: 16/12/2016
*  
*      Descrizione: Definizione dei metodi di accesso alle API OpenWeatherMap. 
*      
*      Riferimento: https://openweathermap.org/api                                     
*/

namespace MappaMeteo
{
    class ExtendedWeatherResult : WeatherResult
    {
        public Double Lon { get; set; }
        public Double Lat { get; set; }

        public Double Clouds { get; set; }
        public Double WindDeg { get; set; }
        public int WeatherId { get; set; }
    }
    static class OpenWeatherMapRequest
    {
        public static string ApiKey { get; set; }
        private static Result<ExtendedWeatherResult> ExtractDataFromResponse(JObject response)
        {
            if (response["cod"].ToString() != "200")
            {
                var errorMessage = "Errore del server " + response["cod"];
                if (!String.IsNullOrEmpty(response["message"].ToString()))
                    errorMessage += ". " + response["message"];
                return new Result<ExtendedWeatherResult>(null, false, errorMessage);
            }

            var list = new List<ExtendedWeatherResult>();

            var responseItems = JArray.Parse(response["list"].ToString());
            foreach (var item in responseItems)
            {
                var current = new ExtendedWeatherResult();

                current.CityId = Convert.ToInt32(item["id"]);
                current.City = Encoding.UTF8.GetString(Encoding.Default.GetBytes(Convert.ToString(item["name"])));

                if (item["coord"] != null)
                {
                    current.Lon = Convert.ToDouble(item["coord"]["lon"]);
                    current.Lat = Convert.ToDouble(item["coord"]["lat"]);
                }

                if (item["main"] != null)
                {
                    current.Temp = Convert.ToDouble(item["main"]["temp"]);
                    current.TempMax = Convert.ToDouble(item["main"]["temp_max"]);
                    current.TempMin = Convert.ToDouble(item["main"]["temp_min"]);
                    current.Humidity = Convert.ToDouble(item["main"]["humidity"]);
                }

                int unixdt = Convert.ToInt32(item["dt"]);
                DateTime temp = new DateTime(1970, 1, 1, 0, 0, 0, 0, System.DateTimeKind.Utc);
                current.Date = temp.AddSeconds(unixdt).ToLocalTime();

                if (item["wind"] != null)
                {
                    current.WindSpeed = Convert.ToDouble(item["wind"]["speed"]);
                    current.WindDeg = Convert.ToDouble(item["wind"]["deg"]);
                }

                if (item["clouds"] != null)
                {
                    current.Clouds = Convert.ToDouble(item["clouds"]["all"]);
                }

                if (item["weather"] != null)
                {
                    current.WeatherId = Convert.ToInt32(item["weather"][0]["id"]);
                    current.Title = Encoding.UTF8.GetString(Encoding.Default.GetBytes(Convert.ToString(item["weather"][0]["main"])));
                    current.Description = Encoding.UTF8.GetString(Encoding.Default.GetBytes(Convert.ToString(item["weather"][0]["description"])));
                    current.Icon = Encoding.UTF8.GetString(Encoding.Default.GetBytes(Convert.ToString(item["weather"][0]["icon"])));
                }

                list.Add(current);
            }

            return new Result<ExtendedWeatherResult>(list, true, "Operazione eseguita correttamente");
        }

        public static Task<Result<ExtendedWeatherResult>> GetCitiesByZone(Double LonLeft, Double LatBottom, Double LonRight, Double LatTop, int Zoom)
        {
            return Task.Run(() =>
            {
                ClientSettings.ApiUrl = "http://api.openweathermap.org/data/2.5";
                ClientSettings.ApiKey = ApiKey;

                string query = String.Format("/box/city?bbox={0},{1},{2},{3},{4}&cluster=yes&lang=it",
                    LonLeft.ToString(CultureInfo.InvariantCulture),
                    LatBottom.ToString(CultureInfo.InvariantCulture),
                    LonRight.ToString(CultureInfo.InvariantCulture),
                    LatTop.ToString(CultureInfo.InvariantCulture),
                    Zoom);

                JObject response;
                Result<ExtendedWeatherResult> result;
                try
                {
                    response = ApiClient.GetResponse(query);
                    result = ExtractDataFromResponse(response);
                }
                catch (Exception ex)
                {
                    result = new Result<ExtendedWeatherResult>(null, false, ex.Message);
                }
                return result;

            });
        }
    }
}
